try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin

import requests

from requests.packages.urllib3.exceptions import InsecureRequestWarning
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

API_URL = "https://mwdb.cert.pl/api/"


class MalwarecageAPI(object):
    def __init__(self, api_url=API_URL, api_key=None):
        self.api_url = api_url
        self.api_key = None
        self.session = requests.Session()
        self.set_api_key(api_key)

    def set_api_key(self, api_key):
        self.api_key = api_key
        self.session.headers.update({'Authorization': 'Bearer {}'.format(self.api_key)})

    def request(self, method, url, noauth=False, raw=False, *args, **kwargs):
        # Check if authenticated yet
        if not noauth and self.api_key is None:
            raise RuntimeError(
                'API credentials for MWDB2 were not set, call MalwarecageAPI.set_api_key or '
                'Malwarecage.login first'
            )

        # Set method name and request URL
        url = urljoin(self.api_url, url)
        # Set default kwargs
        kwargs["verify"] = False
        kwargs["json"] = kwargs.get("json", True)
        response = self.session.request(method, url, *args, **kwargs)
        response.raise_for_status()
        return response.json() if not raw else response.content

    def get(self, *args, **kwargs):
        return self.request("get", *args, **kwargs)

    def post(self, *args, **kwargs):
        return self.request("post", *args, **kwargs)

    def put(self, *args, **kwargs):
        return self.request("put", *args, **kwargs)

    def delete(self, *args, **kwargs):
        return self.request("delete", *args, **kwargs)



import itertools
import json

import requests

from .api import MalwarecageAPI
from .obj import MalwarecageObject
from .file import MalwarecageFile
from .config import MalwarecageConfig
from .blob import MalwarecageBlob


class Malwarecage(object):
    def __init__(self, api=None):
        self.api = api or MalwarecageAPI()

    def login(self, username, password):
        result = self.api.post("auth/login", json={
            "login": username,
            "password": password
        }, noauth=True)
        self.api.set_api_key(result["token"])

    def _recent(self, endpoint):
        try:
            for page in itertools.count(start=1):
                result = self.api.get(endpoint, params={"page": page})
                key = endpoint+"s"
                if key not in result or len(result[key]) == 0:
                    return
                for obj in result[key]:
                    yield MalwarecageObject.create(self.api, obj)
        except requests.exceptions.HTTPError as e:
            if e.response.status_code >= 500:
                raise e

    def recent_objects(self):
        return self._recent("object")

    def recent_files(self):
        return self._recent("file")

    def recent_configs(self):
        return self._recent("config")

    def recent_blobs(self):
        return self._recent("blob")

    def query(self, hash):
        result = self.api.get("object/{}".format(hash))
        return MalwarecageObject.create(self.api, result)

    def query_file(self, hash):
        result = self.api.get("file/{}".format(hash))
        return MalwarecageFile(self.api, result)

    def query_config(self, hash):
        result = self.api.get("config/{}".format(hash))
        return MalwarecageConfig(self.api, result)

    def query_blob(self, hash):
        result = self.api.get("blob/{}".format(hash))
        return MalwarecageConfig(self.api, result)

    def search(self, query):
        result = self.api.post("search", json={"query": query})
        for file in result:
            yield MalwarecageObject.create(self.api, file)

    def upload_file(self, name, content, parent=None, metakeys=None):
        parent = parent or "root"
        result = self.api.post("file/{}".format(parent), data={
            'metakeys': json.dumps({'metakeys': metakeys or []})
        }, files={'file': (name, content)})
        return MalwarecageFile(self.api, result)

    def upload_config(self, family, cfg, parent=None):
        parent = parent or "root"
        result = self.api.put("config/{}".format(parent), json={
            "family": family,
            "cfg": cfg
        })
        return MalwarecageConfig(self.api, result)

    def upload_blob(self, name, type, content, parent=None):
        parent = parent or "root"
        result = self.api.put("blob/{}".format(parent), json={
            "blob_name": name,
            "blob_type": type,
            "content": content
        })
        return MalwarecageBlob(self.api, result)



from .obj import MalwarecageObject, lazy_property


class MalwarecageFile(MalwarecageObject):
    URL_PATTERN = "file/{id}"

    @lazy_property()
    def md5(self):
        return self.data.get("md5")

    @lazy_property()
    def sha1(self):
        return self.data.get("sha1")

    @lazy_property()
    def sha256(self):
        return self.data.get("sha256")

    @lazy_property()
    def sha512(self):
        return self.data.get("sha512")

    @lazy_property()
    def crc32(self):
        return self.data.get("crc32")

    @lazy_property()
    def ssdeep(self):
        return self.data.get("ssdeep")

    @lazy_property()
    def file_name(self):
        return self.data.get("file_name")

    @lazy_property()
    def file_size(self):
        return self.data.get("file_size")

    @lazy_property()
    def file_type(self):
        return self.data.get("file_type")

    def download_content(self):
        token = self.api.post("request/sample/{id}".format(**self.data))["url"].split("/")[-1]
        return self.api.get("download/{}".format(token), raw=True)



from collections import defaultdict
from functools import wraps


def lazy_property(url_pattern=None):
    def wrapper(f):
        @property
        @wraps(f)
        def wrapped_property(self):
            url = (url_pattern or getattr(self, "URL_PATTERN", None)).format(**self.data)
            property = f.__name__
            mapper = getattr(self, "mapper_{}".format(property), lambda d: d)
            if f(self) is None:
                data = self.api.get(url)
                self._update(mapper(data))
            return f(self)
        return wrapped_property
    return wrapper


class MalwarecageElement(object):
    def __init__(self, api, data):
        self.api = api
        self.data = {}
        self._update(data)

    def _update(self, data):
        self.data.update(data)

    @property
    def id(self):
        return self.data["id"]


class MalwarecageComment(MalwarecageElement):
    def __init__(self, api, data, parent):
        super(MalwarecageComment, self).__init__(api, data)
        self.parent = parent

    @property
    def author(self):
        return self.data["author"]

    @property
    def timestamp(self):
        return self.data["timestamp"]

    @property
    def comment(self):
        return self.data["comment"]

    def delete(self):
        self.api.delete("object/{}/comment/{}".format(self.parent.id, self.id))


class MalwarecageObject(MalwarecageElement):
    URL_PATTERN = "object/{id}"

    @staticmethod
    def create(api, data):
        from .file import MalwarecageFile
        from .config import MalwarecageConfig
        from .blob import MalwarecageBlob
        type = data["type"]
        data = {k: v for k, v in data.items() if k != "type"}
        if type == "file":
            return MalwarecageFile(api, data)
        elif type == "static_config":
            return MalwarecageConfig(api, data)
        elif type == "text_blob":
            return MalwarecageBlob(api, data)
        else:
            return None

    def mapper_tags(self, data):
        return {"tags": data}

    @lazy_property("object/{id}/tag")
    def tags(self):
        return [t["tag"] for t in self.data["tags"]] if "tags" in self.data else None

    def mapper_comments(self, data):
        return {"comments": data}

    @lazy_property("object/{id}/comment")
    def comments(self):
        return list(map(lambda c: MalwarecageComment(self.api, c, self), self.data["comments"])) \
            if "comments" in self.data else None

    @lazy_property("object/{id}/meta")
    def metakeys(self):
        if "metakeys" not in self.data:
            return None
        result = defaultdict(list)
        for m in self.data["metakeys"]:
            result[m["key"]].append(m["value"])
        return dict(result)

    @lazy_property()
    def upload_time(self):
        import dateutil.parser
        return dateutil.parser.parse(self.data["upload_time"]) if "upload_time" in self.data else None

    @lazy_property()
    def parents(self):
        return list(map(lambda o: MalwarecageObject.create(self.api, o), self.data["parents"])) \
            if "parents" in self.data else None

    @lazy_property()
    def children(self):
        return list(map(lambda o: MalwarecageObject.create(self.api, o), self.data["children"])) \
            if "children" in self.data else None

    def add_child(self, child):
        self.api.put("object/{parent}/child/{child}".format(parent=self.id, child=child.id))
        if "children" in self.data:
            del self.data["children"]

    def add_tag(self, tag):
        self.api.put("object/{id}/tag".format(**self.data), json={
            "tag": tag
        })
        if "tags" in self.data:
            del self.data["tags"]

    def remove_tag(self, tag):
        self.api.delete("object/{id}/tag".format(**self.data), params={
            "tag": tag
        })
        if "tags" in self.data:
            del self.data["tags"]

    def add_comment(self, comment):
        self.api.post("object/{id}/comment".format(**self.data), json={
            "comment": comment
        })
        if "comments" in self.data:
            del self.data["comments"]

    def add_metakey(self, key, value):
        self.api.post("object/{id}/meta".format(**self.data), json={
            "key": key,
            "value": value
        })

    def flush(self):
        self.data = {"id": self.data["id"]}



from .obj import MalwarecageObject, lazy_property


class MalwarecageBlob(MalwarecageObject):
    URL_PATTERN = "blob/{id}"

    @lazy_property()
    def blob_name(self):
        return self.data.get("blob_name")

    @lazy_property()
    def blob_size(self):
        return self.data.get("blob_size")

    @lazy_property()
    def blob_type(self):
        return self.data.get("blob_type")

    @lazy_property()
    def content(self):
        return self.data.get("content")

    @lazy_property()
    def last_seen(self):
        import dateutil.parser
        return dateutil.parser.parse(self.data["last_seen"]) if "last_seen" in self.data else None



#!/usr/bin/env python

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(name="mwdblib",
      version="2.2.2",
      description="malwaredb API bindings for Python",
      author="psrok1",
      package_dir={'mwdblib': 'src'},
      packages=['mwdblib'],
      url="https://github.com/CERT-Polska/mwdblib",
      install_requires=open("requirements.txt").read().splitlines(),
      classifiers=[
        "Programming Language :: Python",
        "Operating System :: OS Independent",
      ])



from .obj import MalwarecageObject, lazy_property


class MalwarecageConfig(MalwarecageObject):
    URL_PATTERN = "config/{id}"

    @lazy_property()
    def family(self):
        return self.data.get("family")

    @lazy_property()
    def cfg(self):
        return self.data.get("cfg")
